from bamboolib import __widgets_version__


JUPYTERLAB_MANAGER_VERSION_MAPPING = {
    "1.0.x": "1.0",
    "1.1.x": "1.0",
    "1.2.x": "1.1",
    "2.x": "2.0",
}


class Version:
    """Helper class to work with version strings"""

    def __init__(self, version_string: str) -> None:
        version_parts = version_string.split(".")
        self.major = version_parts[0]
        self.minor = version_parts[1]
        self.major_minor = self.major + "." + self.minor


def install_nbextensions() -> None:
    print("Trying to install bamboolib nbextension...")

    try:
        from notebook import nbextensions
    except ImportError:
        print(
            "Could not install bamboolib Jupyter Notebook extension because Jupyter Notebook is not available"
        )
        return

    extensions = ["bamboolib", "qgrid", "widgetsnbextension", "plotlywidget"]
    for extension in extensions:
        nbextensions.install_nbextension_python(extension, user=True)
        nbextensions.enable_nbextension_python(extension)

    print("Finished installing the bamboolib Jupyter Notebook nbextension")
    print("Please reload your Jupyter notebook browser window")


def install_labextensions() -> None:
    print("Trying to install bamboolib labextensions...")

    try:
        from jupyterlab import commands
    except ImportError:
        print(
            "Could not install bamboolib Jupyter Lab extension because Jupyter Lab is not available"
        )
        return

    def jupyterlab_version() -> str:
        return commands.get_app_info()["version"]

    def get_jupyterlab_version_pattern(version: str) -> str:
        version_object = Version(version)

        major_minor_version = version_object.major_minor
        if major_minor_version == "1.0":
            return "1.0.x"
        elif major_minor_version == "1.1":
            return "1.1.x"
        elif major_minor_version == "1.2":
            return "1.2.x"

        major_version = version_object.major
        if major_version == "2":
            return "2.x"

        raise Exception(
            "bamboolib does not support JupyterLab version %s. bamboolib only supports JupyterLab>=1.0. If you have troubles, please reach out to support@8080labs.com"
            % version
        )

    def get_required_jupyterlab_manager_version():
        jl_version = jupyterlab_version()
        jl_version_pattern = get_jupyterlab_version_pattern(jl_version)
        return JUPYTERLAB_MANAGER_VERSION_MAPPING[jl_version_pattern]

    def get_required_plotly_version():
        from plotly import __version__

        return __version__

    extensions = [
        "@jupyter-widgets/jupyterlab-manager@%s"
        % get_required_jupyterlab_manager_version(),
        # "@8080labs/qgrid",
        "qgrid2",
        "plotlywidget@%s" % get_required_plotly_version(),
        "jupyterlab-plotly@%s" % get_required_plotly_version(),
        "bamboolib@%s" % __widgets_version__,
    ]
    for extension in extensions:
        print("Installing %s: ..." % extension, end=" ")
        commands.install_extension(extension)
        print("done")

    print("Starting JupyterLab build. This may take a while ...")
    commands.build()
    print("Successfully built JupyterLab")
    print("")
    print("Please reload your Jupyter Lab browser window")


def install_extensions() -> None:
    print(
        "Starting to install bamboolib extensions for Jupyter Notebook and Jupyter Lab"
    )
    print("")
    install_nbextensions()
    print("")
    install_labextensions()
    print("")
    print("Finished installing the bamboolib Jupyter extensions")
    print("Please reload your Jupyter notebook and/or Jupyter lab browser windows")
